@@ -85,20 +85,10 @@
 #endif
 #include "duktape.h"
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-/* Defined in duk_cmdline_ajduk.c or alljoyn.js headers. */
-void ajsheap_init(void);
-void ajsheap_free(void);
-void ajsheap_dump(void);
-void ajsheap_register(duk_context *ctx);
-void ajsheap_start_exec_timeout(void);
-void ajsheap_clear_exec_timeout(void);
-void *ajsheap_alloc_wrapped(void *udata, duk_size_t size);
-void *ajsheap_realloc_wrapped(void *udata, void *ptr, duk_size_t size);
-void ajsheap_free_wrapped(void *udata, void *ptr);
-void *AJS_Alloc(void *udata, duk_size_t size);
-void *AJS_Realloc(void *udata, void *ptr, duk_size_t size);
-void AJS_Free(void *udata, void *ptr);
+#include "duk_cmdline.h"
+
+#if defined(DUK_CMDLINE_LOWMEM)
+#include "duk_alloc_pool.h"
 #endif
 
 #if defined(DUK_CMDLINE_DEBUGGER_SUPPORT)
@@ -218,7 +208,7 @@ static void print_pop_error(duk_context *ctx, FILE *f) {
 static duk_ret_t wrapped_compile_execute(duk_context *ctx, void *udata) {
 	const char *src_data;
 	duk_size_t src_len;
-	int comp_flags;
+	duk_uint_t comp_flags;
 
 	(void) udata;
 
@@ -237,7 +227,7 @@ static duk_ret_t wrapped_compile_execute(duk_context *ctx, void *udata) {
 	src_data = (const char *) duk_require_pointer(ctx, -3);
 	src_len = (duk_size_t) duk_require_uint(ctx, -2);
 
-	if (src_data != NULL && src_len >= 2 && src_data[0] == (char) 0xff) {
+	if (src_data != NULL && src_len >= 1 && src_data[0] == (char) 0xbf) {
 		/* Bytecode. */
 		void *buf;
 		buf = duk_push_fixed_buffer(ctx, src_len);
@@ -277,12 +267,12 @@ static duk_ret_t wrapped_compile_execute(duk_context *ctx, void *udata) {
 
 		f = fopen(fnbuf, "wb");
 		if (!f) {
-			duk_error(ctx, DUK_ERR_ERROR, "failed to open bytecode output file");
+			(void) duk_generic_error(ctx, "failed to open bytecode output file");
 		}
 		wrote = fwrite(bc_ptr, 1, (size_t) bc_len, f);  /* XXX: handle partial writes */
 		(void) fclose(f);
 		if (wrote != bc_len) {
-			duk_error(ctx, DUK_ERR_ERROR, "failed to write all bytecode");
+			(void) duk_generic_error(ctx, "failed to write all bytecode");
 		}
 
 		return 0;  /* duk_safe_call() cleans up */
@@ -297,15 +287,15 @@ static duk_ret_t wrapped_compile_execute(duk_context *ctx, void *udata) {
 	duk_load_function(ctx);
 #endif
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	ajsheap_start_exec_timeout();
+#if defined(DUK_CMDLINE_LOWMEM)
+	lowmem_start_exec_timeout();
 #endif
 
 	duk_push_global_object(ctx);  /* 'this' binding */
 	duk_call_method(ctx, 0);
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	ajsheap_clear_exec_timeout();
+#if defined(DUK_CMDLINE_LOWMEM)
+	lowmem_clear_exec_timeout();
 #endif
 
 	if (interactive_mode) {
@@ -635,8 +625,8 @@ static int handle_fh(duk_context *ctx, FILE *f, const char *filename, const char
 
 	rc = duk_safe_call(ctx, wrapped_compile_execute, NULL /*udata*/, 4 /*nargs*/, 1 /*nret*/);
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	ajsheap_clear_exec_timeout();
+#if defined(DUK_CMDLINE_LOWMEM)
+	lowmem_clear_exec_timeout();
 #endif
 
 	free(buf);
@@ -713,8 +703,8 @@ static int handle_eval(duk_context *ctx, char *code) {
 
 	rc = duk_safe_call(ctx, wrapped_compile_execute, NULL /*udata*/, 3 /*nargs*/, 1 /*nret*/);
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	ajsheap_clear_exec_timeout();
+#if defined(DUK_CMDLINE_LOWMEM)
+	lowmem_clear_exec_timeout();
 #endif
 
 	if (rc != DUK_EXEC_SUCCESS) {
@@ -778,8 +768,8 @@ static int handle_interactive(duk_context *ctx) {
 
 		rc = duk_safe_call(ctx, wrapped_compile_execute, NULL /*udata*/, 3 /*nargs*/, 1 /*nret*/);
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-		ajsheap_clear_exec_timeout();
+#if defined(DUK_CMDLINE_LOWMEM)
+		lowmem_clear_exec_timeout();
 #endif
 
 		if (buffer) {
@@ -850,8 +840,8 @@ static int handle_interactive(duk_context *ctx) {
 
 		rc = duk_safe_call(ctx, wrapped_compile_execute, NULL /*udata*/, 3 /*nargs*/, 1 /*nret*/);
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-		ajsheap_clear_exec_timeout();
+#if defined(DUK_CMDLINE_LOWMEM)
+		lowmem_clear_exec_timeout();
 #endif
 
 		if (rc != DUK_EXEC_SUCCESS) {
@@ -889,22 +879,22 @@ static duk_ret_t fileio_read_file(duk_context *ctx) {
 	fn = duk_require_string(ctx, 0);
 	f = fopen(fn, "rb");
 	if (!f) {
-		duk_error(ctx, DUK_ERR_TYPE_ERROR, "cannot open file %s for reading, errno %ld: %s",
-		          fn, (long) errno, strerror(errno));
+		(void) duk_type_error(ctx, "cannot open file %s for reading, errno %ld: %s",
+		                      fn, (long) errno, strerror(errno));
 	}
 
 	rc = fseek(f, 0, SEEK_END);
 	if (rc < 0) {
 		(void) fclose(f);
-		duk_error(ctx, DUK_ERR_TYPE_ERROR, "fseek() failed for %s, errno %ld: %s",
-		          fn, (long) errno, strerror(errno));
+		(void) duk_type_error(ctx, "fseek() failed for %s, errno %ld: %s",
+		                      fn, (long) errno, strerror(errno));
 	}
 	len = (size_t) ftell(f);
 	rc = fseek(f, 0, SEEK_SET);
 	if (rc < 0) {
 		(void) fclose(f);
-		duk_error(ctx, DUK_ERR_TYPE_ERROR, "fseek() failed for %s, errno %ld: %s",
-		          fn, (long) errno, strerror(errno));
+		(void) duk_type_error(ctx, "fseek() failed for %s, errno %ld: %s",
+		                      fn, (long) errno, strerror(errno));
 	}
 
 	buf = (char *) duk_push_fixed_buffer(ctx, (duk_size_t) len);
@@ -913,14 +903,14 @@ static duk_ret_t fileio_read_file(duk_context *ctx) {
 		got = fread((void *) (buf + off), 1, len - off, f);
 		if (ferror(f)) {
 			(void) fclose(f);
-			duk_error(ctx, DUK_ERR_TYPE_ERROR, "error while reading %s", fn);
+			(void) duk_type_error(ctx, "error while reading %s", fn);
 		}
 		if (got == 0) {
 			if (feof(f)) {
 				break;
 			} else {
 				(void) fclose(f);
-				duk_error(ctx, DUK_ERR_TYPE_ERROR, "error while reading %s", fn);
+				(void) duk_type_error(ctx, "error while reading %s", fn);
 			}
 		}
 		off += got;
@@ -943,7 +933,7 @@ static duk_ret_t fileio_write_file(duk_context *ctx) {
 	fn = duk_require_string(ctx, 0);
 	f = fopen(fn, "wb");
 	if (!f) {
-		duk_error(ctx, DUK_ERR_TYPE_ERROR, "cannot open file %s for writing, errno %ld: %s",
+		(void) duk_type_error(ctx, "cannot open file %s for writing, errno %ld: %s",
 		          fn, (long) errno, strerror(errno));
 	}
 
@@ -954,11 +944,11 @@ static duk_ret_t fileio_write_file(duk_context *ctx) {
 		got = fwrite((const void *) (buf + off), 1, len - off, f);
 		if (ferror(f)) {
 			(void) fclose(f);
-			duk_error(ctx, DUK_ERR_TYPE_ERROR, "error while writing %s", fn);
+			(void) duk_type_error(ctx, "error while writing %s", fn);
 		}
 		if (got == 0) {
 			(void) fclose(f);
-			duk_error(ctx, DUK_ERR_TYPE_ERROR, "error while writing %s", fn);
+			(void) duk_type_error(ctx, "error while writing %s", fn);
 		}
 		off += got;
 	}
@@ -1051,12 +1041,12 @@ static void debugger_detached(duk_context *ctx, void *udata) {
 #define  ALLOC_LOGGING  1
 #define  ALLOC_TORTURE  2
 #define  ALLOC_HYBRID   3
-#define  ALLOC_AJSHEAP  4
+#define  ALLOC_LOWMEM   4
 
-static duk_context *create_duktape_heap(int alloc_provider, int debugger, int ajsheap_log) {
+static duk_context *create_duktape_heap(int alloc_provider, int debugger, int lowmem_log) {
 	duk_context *ctx;
 
-	(void) ajsheap_log;  /* suppress warning */
+	(void) lowmem_log;  /* suppress warning */
 
 	ctx = NULL;
 	if (!ctx && alloc_provider == ALLOC_LOGGING) {
@@ -1101,15 +1091,15 @@ static duk_context *create_duktape_heap(int alloc_provider, int debugger, int aj
 		fflush(stderr);
 #endif
 	}
-	if (!ctx && alloc_provider == ALLOC_AJSHEAP) {
-#if defined(DUK_CMDLINE_AJSHEAP)
-		ajsheap_init();
+	if (!ctx && alloc_provider == ALLOC_LOWMEM) {
+#if defined(DUK_CMDLINE_LOWMEM)
+		lowmem_init();
 
 		ctx = duk_create_heap(
-			ajsheap_log ? ajsheap_alloc_wrapped : AJS_Alloc,
-			ajsheap_log ? ajsheap_realloc_wrapped : AJS_Realloc,
-			ajsheap_log ? ajsheap_free_wrapped : AJS_Free,
-			(void *) 0xdeadbeef,  /* heap_udata: ignored by AjsHeap, use as marker */
+			lowmem_log ? lowmem_alloc_wrapped : duk_alloc_pool,
+			lowmem_log ? lowmem_realloc_wrapped : duk_realloc_pool,
+			lowmem_log ? lowmem_free_wrapped : duk_free_pool,
+			(void *) lowmem_pool_ptr,
 			cmdline_fatal_handler);
 #else
 		fprintf(stderr, "Warning: option --alloc-ajsheap ignored, no ajsheap allocator support\n");
@@ -1126,16 +1116,16 @@ static duk_context *create_duktape_heap(int alloc_provider, int debugger, int aj
 		exit(1);
 	}
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	if (alloc_provider == ALLOC_AJSHEAP) {
-		fprintf(stdout, "Pool dump after heap creation\n");
-		ajsheap_dump();
+#if defined(DUK_CMDLINE_LOWMEM)
+	if (alloc_provider == ALLOC_LOWMEM) {
+		fprintf(stderr, "*** pool dump after heap creation ***\n");
+		lowmem_dump();
 	}
 #endif
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	if (alloc_provider == ALLOC_AJSHEAP) {
-		ajsheap_register(ctx);
+#if defined(DUK_CMDLINE_LOWMEM)
+	if (alloc_provider == ALLOC_LOWMEM) {
+		lowmem_register(ctx);
 	}
 #endif
 
@@ -1161,7 +1151,7 @@ static duk_context *create_duktape_heap(int alloc_provider, int debugger, int aj
 
 	/* Register console object. */
 #if defined(DUK_CMDLINE_CONSOLE_SUPPORT)
-	duk_console_init(ctx, DUK_CONSOLE_PROXY_WRAPPER /*flags*/);
+	duk_console_init(ctx, DUK_CONSOLE_PROXY_WRAPPER | DUK_CONSOLE_FLUSH /*flags*/);
 #endif
 
 	/* Register Duktape.Logger (removed in Duktape 2.x). */
@@ -1237,15 +1227,15 @@ static duk_context *create_duktape_heap(int alloc_provider, int debugger, int aj
 static void destroy_duktape_heap(duk_context *ctx, int alloc_provider) {
 	(void) alloc_provider;
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	if (alloc_provider == ALLOC_AJSHEAP) {
-		fprintf(stdout, "Pool dump before duk_destroy_heap(), before forced gc\n");
-		ajsheap_dump();
+#if defined(DUK_CMDLINE_LOWMEM)
+	if (alloc_provider == ALLOC_LOWMEM) {
+		fprintf(stderr, "*** pool dump before duk_destroy_heap(), before forced gc ***\n");
+		lowmem_dump();
 
 		duk_gc(ctx, 0);
 
-		fprintf(stdout, "Pool dump before duk_destroy_heap(), after forced gc\n");
-		ajsheap_dump();
+		fprintf(stderr, "*** pool dump before duk_destroy_heap(), after forced gc ***\n");
+		lowmem_dump();
 	}
 #endif
 
@@ -1253,12 +1243,12 @@ static void destroy_duktape_heap(duk_context *ctx, int alloc_provider) {
 		duk_destroy_heap(ctx);
 	}
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	if (alloc_provider == ALLOC_AJSHEAP) {
-		fprintf(stdout, "Pool dump after duk_destroy_heap() (should have zero allocs)\n");
-		ajsheap_dump();
+#if defined(DUK_CMDLINE_LOWMEM)
+	if (alloc_provider == ALLOC_LOWMEM) {
+		fprintf(stderr, "*** pool dump after duk_destroy_heap() (should have zero allocs) ***\n");
+		lowmem_dump();
 	}
-	ajsheap_free();
+	lowmem_free();
 #endif
 }
 
@@ -1274,7 +1264,7 @@ int main(int argc, char *argv[]) {
 	int interactive = 0;
 	int memlimit_high = 1;
 	int alloc_provider = ALLOC_DEFAULT;
-	int ajsheap_log = 0;
+	int lowmem_log = 0;
 	int debugger = 0;
 	int recreate_heap = 0;
 	int no_heap_destroy = 0;
@@ -1333,10 +1323,10 @@ int main(int argc, char *argv[]) {
 	);
 #endif  /* EMSCRIPTEN */
 
-#if defined(DUK_CMDLINE_AJSHEAP)
-	alloc_provider = ALLOC_AJSHEAP;
+#if defined(DUK_CMDLINE_LOWMEM)
+	alloc_provider = ALLOC_LOWMEM;
 #endif
-	(void) ajsheap_log;
+	(void) lowmem_log;
 
 	/*
 	 *  Signal handling setup
@@ -1382,10 +1372,10 @@ int main(int argc, char *argv[]) {
 			alloc_provider = ALLOC_TORTURE;
 		} else if (strcmp(arg, "--alloc-hybrid") == 0) {
 			alloc_provider = ALLOC_HYBRID;
-		} else if (strcmp(arg, "--alloc-ajsheap") == 0) {
-			alloc_provider = ALLOC_AJSHEAP;
-		} else if (strcmp(arg, "--ajsheap-log") == 0) {
-			ajsheap_log = 1;
+		} else if (strcmp(arg, "--alloc-lowmem") == 0) {
+			alloc_provider = ALLOC_LOWMEM;
+		} else if (strcmp(arg, "--lowmem-log") == 0) {
+			lowmem_log = 1;
 		} else if (strcmp(arg, "--debugger") == 0) {
 			debugger = 1;
 #if defined(DUK_CMDLINE_DEBUGGER_SUPPORT)
@@ -1427,7 +1417,7 @@ int main(int argc, char *argv[]) {
 	 *  Create heap
 	 */
 
-	ctx = create_duktape_heap(alloc_provider, debugger, ajsheap_log);
+	ctx = create_duktape_heap(alloc_provider, debugger, lowmem_log);
 
 	/*
 	 *  Execute any argument file(s)
@@ -1473,7 +1463,7 @@ int main(int argc, char *argv[]) {
 			}
 
 			destroy_duktape_heap(ctx, alloc_provider);
-			ctx = create_duktape_heap(alloc_provider, debugger, ajsheap_log);
+			ctx = create_duktape_heap(alloc_provider, debugger, lowmem_log);
 		}
 	}
 
@@ -1494,7 +1484,7 @@ int main(int argc, char *argv[]) {
 			}
 
 			destroy_duktape_heap(ctx, alloc_provider);
-			ctx = create_duktape_heap(alloc_provider, debugger, ajsheap_log);
+			ctx = create_duktape_heap(alloc_provider, debugger, lowmem_log);
 		}
 	}
 
@@ -1545,7 +1535,7 @@ int main(int argc, char *argv[]) {
 	                "   --restrict-memory  use lower memory limit (used by test runner)\n"
 	                "   --alloc-default    use Duktape default allocator\n"
 #if defined(DUK_CMDLINE_ALLOC_LOGGING)
-	                "   --alloc-logging    use logging allocator (writes to /tmp)\n"
+	                "   --alloc-logging    use logging allocator, write alloc log to /tmp/duk-alloc-log.txt\n"
 #endif
 #if defined(DUK_CMDLINE_ALLOC_TORTURE)
 	                "   --alloc-torture    use torture allocator\n"
@@ -1553,9 +1543,9 @@ int main(int argc, char *argv[]) {
 #if defined(DUK_CMDLINE_ALLOC_HYBRID)
 	                "   --alloc-hybrid     use hybrid allocator\n"
 #endif
-#if defined(DUK_CMDLINE_AJSHEAP)
-	                "   --alloc-ajsheap    use ajsheap allocator (enabled by default with 'ajduk')\n"
-	                "   --ajsheap-log      write alloc log to /tmp/ajduk-alloc-log.txt\n"
+#if defined(DUK_CMDLINE_LOWMEM)
+	                "   --alloc-lowmem     use pooled allocator (enabled by default for duk-low)\n"
+	                "   --lowmem-log       write alloc log to /tmp/lowmem-alloc-log.txt\n"
 #endif
 #if defined(DUK_CMDLINE_DEBUGGER_SUPPORT)
 			"   --debugger         start example debugger\n"
