@@ -12,10 +12,10 @@ struct duk__compile_raw_args {
 };
 
 /* Eval is just a wrapper now. */
-DUK_EXTERNAL duk_int_t duk_eval_raw(duk_context *ctx, const char *src_buffer, duk_size_t src_length, duk_uint_t flags) {
+DUK_EXTERNAL duk_int_t duk_eval_raw(duk_hthread *thr, const char *src_buffer, duk_size_t src_length, duk_uint_t flags) {
 	duk_int_t rc;
 
-	DUK_ASSERT_CTX_VALID(ctx);
+	DUK_ASSERT_API_ENTRY(thr);
 
 	/* Note: strictness is *not* inherited from the current Duktape/C.
 	 * This would be confusing because the current strictness state
@@ -26,7 +26,7 @@ DUK_EXTERNAL duk_int_t duk_eval_raw(duk_context *ctx, const char *src_buffer, du
 
 	/* [ ... source? filename? ] (depends on flags) */
 
-	rc = duk_compile_raw(ctx, src_buffer, src_length, flags | DUK_COMPILE_EVAL);  /* may be safe, or non-safe depending on flags */
+	rc = duk_compile_raw(thr, src_buffer, src_length, flags | DUK_COMPILE_EVAL);  /* may be safe, or non-safe depending on flags */
 
 	/* [ ... closure/error ] */
 
@@ -35,12 +35,12 @@ DUK_EXTERNAL duk_int_t duk_eval_raw(duk_context *ctx, const char *src_buffer, du
 		goto got_rc;
 	}
 
-	duk_push_global_object(ctx);  /* explicit 'this' binding, see GH-164 */
+	duk_push_global_object(thr);  /* explicit 'this' binding, see GH-164 */
 
 	if (flags & DUK_COMPILE_SAFE) {
-		rc = duk_pcall_method(ctx, 0);
+		rc = duk_pcall_method(thr, 0);
 	} else {
-		duk_call_method(ctx, 0);
+		duk_call_method(thr, 0);
 		rc = DUK_EXEC_SUCCESS;
 	}
 
@@ -48,20 +48,19 @@ DUK_EXTERNAL duk_int_t duk_eval_raw(duk_context *ctx, const char *src_buffer, du
 
  got_rc:
 	if (flags & DUK_COMPILE_NORESULT) {
-		duk_pop(ctx);
+		duk_pop(thr);
 	}
 
 	return rc;
 }
 
 /* Helper which can be called both directly and with duk_safe_call(). */
-DUK_LOCAL duk_ret_t duk__do_compile(duk_context *ctx, void *udata) {
-	duk_hthread *thr = (duk_hthread *) ctx;
+DUK_LOCAL duk_ret_t duk__do_compile(duk_hthread *thr, void *udata) {
 	duk__compile_raw_args *comp_args;
 	duk_uint_t flags;
 	duk_hcompfunc *h_templ;
 
-	DUK_ASSERT_CTX_VALID(ctx);
+	DUK_ASSERT_CTX_VALID(thr);
 	DUK_ASSERT(udata != NULL);
 
 	/* Note: strictness is not inherited from the current Duktape/C
@@ -78,7 +77,7 @@ DUK_LOCAL duk_ret_t duk__do_compile(duk_context *ctx, void *udata) {
 
 	if (flags & DUK_COMPILE_NOFILENAME) {
 		/* Automatic filename: 'eval' or 'input'. */
-		duk_push_hstring_stridx(ctx, (flags & DUK_COMPILE_EVAL) ? DUK_STRIDX_EVAL : DUK_STRIDX_INPUT);
+		duk_push_hstring_stridx(thr, (flags & DUK_COMPILE_EVAL) ? DUK_STRIDX_EVAL : DUK_STRIDX_INPUT);
 	}
 
 	/* [ ... source? filename ] */
@@ -86,7 +85,7 @@ DUK_LOCAL duk_ret_t duk__do_compile(duk_context *ctx, void *udata) {
 	if (!comp_args->src_buffer) {
 		duk_hstring *h_sourcecode;
 
-		h_sourcecode = duk_get_hstring(ctx, -2);
+		h_sourcecode = duk_get_hstring(thr, -2);
 		if ((flags & DUK_COMPILE_NOSOURCE) ||  /* args incorrect */
 		    (h_sourcecode == NULL)) {          /* e.g. duk_push_string_file_raw() pushed undefined */
 			DUK_ERROR_TYPE(thr, DUK_STR_NO_SOURCECODE);
@@ -110,29 +109,29 @@ DUK_LOCAL duk_ret_t duk__do_compile(duk_context *ctx, void *udata) {
 	if (flags & DUK_COMPILE_NOSOURCE) {
 		;
 	} else {
-		duk_remove_m2(ctx);
+		duk_remove_m2(thr);
 	}
 
 	/* [ ... func_template ] */
 
-	h_templ = (duk_hcompfunc *) duk_known_hobject(ctx, -1);
+	h_templ = (duk_hcompfunc *) duk_known_hobject(thr, -1);
 	duk_js_push_closure(thr,
 	                   h_templ,
 	                   thr->builtins[DUK_BIDX_GLOBAL_ENV],
 	                   thr->builtins[DUK_BIDX_GLOBAL_ENV],
 	                   1 /*add_auto_proto*/);
-	duk_remove_m2(ctx);   /* -> [ ... closure ] */
+	duk_remove_m2(thr);   /* -> [ ... closure ] */
 
 	/* [ ... closure ] */
 
 	return 1;
 }
 
-DUK_EXTERNAL duk_int_t duk_compile_raw(duk_context *ctx, const char *src_buffer, duk_size_t src_length, duk_uint_t flags) {
+DUK_EXTERNAL duk_int_t duk_compile_raw(duk_hthread *thr, const char *src_buffer, duk_size_t src_length, duk_uint_t flags) {
 	duk__compile_raw_args comp_args_alloc;
 	duk__compile_raw_args *comp_args = &comp_args_alloc;
 
-	DUK_ASSERT_CTX_VALID(ctx);
+	DUK_ASSERT_API_ENTRY(thr);
 
 	if ((flags & DUK_COMPILE_STRLEN) && (src_buffer != NULL)) {
 		/* String length is computed here to avoid multiple evaluation
@@ -159,13 +158,13 @@ DUK_EXTERNAL duk_int_t duk_compile_raw(duk_context *ctx, const char *src_buffer,
 		nargs = flags & 0x07;
 		DUK_ASSERT(nargs == ((flags & DUK_COMPILE_NOSOURCE) ? 0 : 1) +
 		                    ((flags & DUK_COMPILE_NOFILENAME) ? 0 : 1));
-		rc = duk_safe_call(ctx, duk__do_compile, (void *) comp_args, nargs, nrets);
+		rc = duk_safe_call(thr, duk__do_compile, (void *) comp_args, nargs, nrets);
 
 		/* [ ... closure ] */
 		return rc;
 	}
 
-	(void) duk__do_compile(ctx, (void *) comp_args);
+	(void) duk__do_compile(thr, (void *) comp_args);
 
 	/* [ ... closure ] */
 	return DUK_EXEC_SUCCESS;
