@@ -235,6 +235,17 @@ extern const duk_uint8_t duk_unicode_caseconv_lc[680];
 extern const duk_uint16_t duk_unicode_re_canon_lookup[65536];
 #endif
 
+#if defined(DUK_USE_REGEXP_CANON_BITMAP)
+/*
+ *  Automatically generated by extract_caseconv.py, do not edit!
+ */
+
+#define DUK_CANON_BITMAP_BLKSIZE                                      32
+#define DUK_CANON_BITMAP_BLKSHIFT                                     5
+#define DUK_CANON_BITMAP_BLKMASK                                      31
+extern const duk_uint8_t duk_unicode_re_canon_bitmap[256];
+#endif
+
 /*
  *  Extern
  */
