@@ -423,11 +423,13 @@ def main():
         'duk_bi_math.c',
         'duk_bi_number.c',
         'duk_bi_object.c',
+        'duk_bi_performance.c',
         'duk_bi_pointer.c',
         'duk_bi_protos.h',
         'duk_bi_reflect.c',
         'duk_bi_regexp.c',
         'duk_bi_string.c',
+        'duk_bi_promise.c',
         'duk_bi_proxy.c',
         'duk_bi_symbol.c',
         'duk_bi_thread.c',
@@ -447,6 +449,7 @@ def main():
         'duk_error_throw.c',
         'duk_forwdecl.h',
         'duk_harray.h',
+        'duk_hboundfunc.h',
         'duk_hbuffer_alloc.c',
         'duk_hbuffer.h',
         'duk_hbuffer_ops.c',
@@ -472,6 +475,7 @@ def main():
         'duk_hobject_misc.c',
         'duk_hobject_pc2line.c',
         'duk_hobject_props.c',
+        'duk_hproxy.h',
         'duk_hstring.h',
         'duk_hstring_misc.c',
         'duk_hthread_alloc.c',
@@ -783,6 +787,20 @@ def main():
         with open(os.path.join(tempdir, 'caseconv_re_canon_lookup.txt'), 'wb') as f:
             f.write(res)
 
+        logger.debug('- extract_caseconv canon bitmap')
+        res = exec_get_stdout([
+            sys.executable,
+            os.path.join(script_path, 'extract_caseconv.py'),
+            '--command=re_canon_bitmap',
+            '--unicode-data', os.path.join(tempdir, 'UnicodeData-expanded.tmp'),
+            '--special-casing', special_casing,
+            '--out-source', os.path.join(tempdir, 'duk_unicode_re_canon_bitmap.c.tmp'),
+            '--out-header', os.path.join(tempdir, 'duk_unicode_re_canon_bitmap.h.tmp'),
+            '--table-name-re-canon-bitmap', 'duk_unicode_re_canon_bitmap'
+        ])
+        with open(os.path.join(tempdir, 'caseconv_re_canon_bitmap.txt'), 'wb') as f:
+            f.write(res)
+
     # XXX: Now with configure.py part of the distributable, could generate
     # only those Unicode tables needed by desired configuration (e.g. BMP-only
     # tables if BMP-only was enabled).
@@ -823,7 +841,8 @@ def main():
         '#include "duk_unicode_idp_m_ids_noa.h"': read_file(os.path.join(tempdir, 'duk_unicode_idp_m_ids_noa.h.tmp'), strip_last_nl=True),
         '#include "duk_unicode_idp_m_ids_noabmp.h"': read_file(os.path.join(tempdir, 'duk_unicode_idp_m_ids_noabmp.h.tmp'), strip_last_nl=True),
         '#include "duk_unicode_caseconv.h"': read_file(os.path.join(tempdir, 'duk_unicode_caseconv.h.tmp'), strip_last_nl=True),
-        '#include "duk_unicode_re_canon_lookup.h"': read_file(os.path.join(tempdir, 'duk_unicode_re_canon_lookup.h.tmp'), strip_last_nl=True)
+        '#include "duk_unicode_re_canon_lookup.h"': read_file(os.path.join(tempdir, 'duk_unicode_re_canon_lookup.h.tmp'), strip_last_nl=True),
+        '#include "duk_unicode_re_canon_bitmap.h"': read_file(os.path.join(tempdir, 'duk_unicode_re_canon_bitmap.h.tmp'), strip_last_nl=True)
     })
 
     copy_and_replace(os.path.join(tempdir, 'src', 'duk_unicode_tables.c'), os.path.join(tempdir, 'src', 'duk_unicode_tables.c'), {
@@ -834,7 +853,8 @@ def main():
         '#include "duk_unicode_idp_m_ids_noa.c"': read_file(os.path.join(tempdir, 'duk_unicode_idp_m_ids_noa.c.tmp'), strip_last_nl=True),
         '#include "duk_unicode_idp_m_ids_noabmp.c"': read_file(os.path.join(tempdir, 'duk_unicode_idp_m_ids_noabmp.c.tmp'), strip_last_nl=True),
         '#include "duk_unicode_caseconv.c"': read_file(os.path.join(tempdir, 'duk_unicode_caseconv.c.tmp'), strip_last_nl=True),
-        '#include "duk_unicode_re_canon_lookup.c"': read_file(os.path.join(tempdir, 'duk_unicode_re_canon_lookup.c.tmp'), strip_last_nl=True)
+        '#include "duk_unicode_re_canon_lookup.c"': read_file(os.path.join(tempdir, 'duk_unicode_re_canon_lookup.c.tmp'), strip_last_nl=True),
+        '#include "duk_unicode_re_canon_bitmap.c"': read_file(os.path.join(tempdir, 'duk_unicode_re_canon_bitmap.c.tmp'), strip_last_nl=True)
     })
 
     # Create a combined source file, duktape.c, into a separate combined source
